package com.loafle.overflow.central.module.target.service;

import java.util.ArrayList;
import java.util.List;

import com.loafle.overflow.central.module.probe.service.CentralProbeService;
import com.loafle.overflow.central.module.sensor.service.CentralSensorService;
import com.loafle.overflow.central.module.target.dao.TargetDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.meta.MetaTargetHostType;
import com.loafle.overflow.model.meta.MetaTargetStatus;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by insanity on 17. 6. 28.
 */
@org.springframework.stereotype.Service("TargetService")
public class CentralTargetService implements TargetService {

  @Autowired
  private TargetDAO targetDAO;
  @Autowired
  private InfraService infraService;
  @Autowired
  private CentralProbeService probeService;
  @Autowired
  private CentralSensorService sensorService;

  @Transactional
  public Target regist(Target target, Long probeID) throws OverflowException {
    this.probeService.increaseTargetCount(probeID);
    return this.targetDAO.save(target);
  }

  @Transactional
  public void remove(Long id, Long probeID) throws OverflowException {
    this.probeService.decreaseTargetCount(probeID);
    this.targetDAO.deleteById(id);
  }

  public Target read(Long id) throws OverflowException {
    return this.targetDAO.findById(id).get();
  }

  public Target modify(Target target) throws OverflowException {
    return this.targetDAO.save(target);
  }

  public Target increaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    t.setSensorCount(t.getSensorCount() + 1);
    return this.targetDAO.save(t);
  }

  public Target decreaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    int count = t.getSensorCount();
    if (t.getSensorCount() > 0) {
      t.setSensorCount(count - 1);
    }
    return this.targetDAO.save(t);
  }

  public List<Target> readAllByProbeID(Long probeID) throws OverflowException {
    return this.targetDAO.findAllByInfraProbeId(probeID);
  }

  @Transactional
  public List<Target> registAll(List<Long> infraIDs, Long probeID) throws OverflowException {
    List<Target> targets = new ArrayList<Target>();
    for (Long infraID : infraIDs) {
      Infra infra = this.infraService.read(infraID);
      if (null == infra) {
        throw new OverflowException(String.format("ID[%d] of Infra is not exist.", infraID));
      }
      String metaInfraTypeKey = infra.getMetaInfraType().getKey();
      Target target = new Target();
      target.setInfra(infra);
      target.setSensorCount(0);
      target.setMetaTargetStatus(MetaTargetStatus.Enum.UP.to());
      if (metaInfraTypeKey.equals("HOST")) {
        InfraHost infraHost = (InfraHost) infra;
        target.setMetaTargetType(MetaTargetHostType.Enum.UNKNOWN.to());
        target.setName(infraHost.getInfraHostIPs().get(0).getAddress());
      } else if (metaInfraTypeKey.equals("SERVICE")) {
        com.loafle.overflow.model.infra.InfraService infraService = (com.loafle.overflow.model.infra.InfraService) infra;
        target.setMetaTargetType(infraService.getMetaTargetServiceType());
        String name = infraService.getMetaTargetServiceType().getName() + " on "
            + infraService.getInfraHostPort().getInfraHostIP().getAddress();
        target.setName(name);
      }
      targets.add(target);
    }
    List<Target> registered = this.targetDAO.saveAll(targets);
    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }
    Integer targetCount = probe.getTargetCount() + registered.size();
    probe.setTargetCount(targetCount);
    this.probeService.modify(probe);

    // FIXME : process asynchronously
    this.sensorService.registDefaults(registered);

    return registered;
  }

}
